using NUnit.Framework;
using System.Linq;

namespace Encompass
{
    namespace Tests {
        class MockComponent : Component {
            public string myString;
            public int myInt;
        }

        public class EntityTest
        {
            [Test]
            public void AddComponent()
            {
                var world = new World();
                var entity = world.CreateEntity();

                var mockComponent = entity.AddComponent<MockComponent>();
                mockComponent.myString = "hello";
                mockComponent.myInt = 3;

                world.Update();

                Assert.IsTrue(entity.HasComponent<MockComponent>());
                Assert.AreEqual(mockComponent, entity.GetComponent<MockComponent>());
            }

            [Test]
            public void GetComponents()
            {
                var world = new World();
                var entity = world.CreateEntity();

                var mockComponentA = entity.AddComponent<MockComponent>();
                var mockComponentB = entity.AddComponent<MockComponent>();
                var mockComponentC = entity.AddComponent<MockComponent>();

                world.Update();

                Assert.Contains(mockComponentA, entity.GetComponents<MockComponent>().ToList());
                Assert.Contains(mockComponentB, entity.GetComponents<MockComponent>().ToList());
                Assert.Contains(mockComponentC, entity.GetComponents<MockComponent>().ToList());
            }

            [Test]
            public void GetComponent()
            {
                var world = new World();
                var entity = world.CreateEntity();

                var mockComponent = entity.AddComponent<MockComponent>();
                mockComponent.myString = "hello";
                mockComponent.myInt = 3;

                world.Update();

                Assert.AreEqual(mockComponent, entity.GetComponent<MockComponent>());
            }

            [Test]
            public void HasComponent()
            {
                var world = new World();
                var entity = world.CreateEntity();

                var mockComponent = entity.AddComponent<MockComponent>();

                world.Update();

                Assert.IsTrue(entity.HasComponent<MockComponent>());
            }
        }
    }
}
